@@ -1,10 +1,11 @@
 from urllib import urlretrieve
 from random import choice
-from string import printable, lowercase, digits
+from string import letters, lowercase, digits
 from random import randint, randrange, random
 from datetime import date, datetime, timedelta
 from decimal import Decimal
 from os import path, listdir
+from tempfile import NamedTemporaryFile
 
 from django.db.models import fields
 from django.core.files import File
@@ -29,13 +30,25 @@ def get_or_create_random(model, **kwargs):
         # Use defaults only to create an instance, not to get an existing one
         params.update(defaults)
         for field in model._meta.fields:
-            if not (field.name in params or field.null or field.blank or \
-                field.has_default()):
+            if not (field.name in params or field_to_skip(field)):
                 params[field.name] = fill_field(field)
         obj = model(**params)
         obj.save()
         return obj
 
+def field_to_skip(field):
+    """
+    Check conditions by which a field should NOT be filled randomly.
+    """
+    skip = field.null or field.blank or field.has_default()
+    # NOTE: The following conditions states that OneToOneField should not be
+    # filled. The reason is that these fields are used for model inheritance
+    # and (in that case) are automatically filled.
+    # FIXME: Check if there are other cases where OneToOneField should be
+    #        filled with random values
+    skip = skip or (type(field) == fields.related.OneToOneField)
+    return skip
+
 def fill_field(field):
     """
     Fill a field with a random content based on its type.
@@ -44,7 +57,7 @@ def fill_field(field):
     try:
         return eval("random_%s(field)" % internal_type)
     except NameError:
-        raise NameError, "Don't know how randomly fill a %s" % internal_type    
+        raise NameError, "Don't know how randomly fill a %s" % internal_type
 
 def random_BigIntegerField(field):
     """
@@ -68,7 +81,10 @@ def random_CharField(field):
             limit = validator.limit_value
     if type(field) == fields.CharField:
         length = xrange(randint(1, limit))
-        return "".join([choice(printable) for i in length])
+        if field.choices:
+            return choice(field.choices)[0]
+        else:
+            return "".join([choice(letters + digits) for i in length])
     elif type(field) == fields.CommaSeparatedIntegerField:
         length = xrange(randint(1, limit))
         return "".join([choice(digits+',') for i in length]).strip(',')
@@ -114,9 +130,10 @@ def random_FileField(field):
     elif type(field) == fields.files.ImageField:
         # TODO: Get a random image from flickr instead
         remote_path="http://blazingwolf.com/drupal6/files/random%20flame.jpg"
-        local_path = 'tmp/random.jpg' # to not clutter the current folder
-        # if not exists(local_path): 
-        urlretrieve(remote_path, local_path)    
+        tmp_file = NamedTemporaryFile(delete=False)
+        tmp_file.close()
+        local_path = tmp_file.name
+        urlretrieve(remote_path, local_path)
     f = open(local_path, 'r')
     f.read()
     # TODO: Close the file at the end
@@ -131,7 +148,7 @@ def random_FilePathField(field):
         folder = field.path
     else:
         folder = "."
-    return choice([f for f in listdir(folder) 
+    return choice([f for f in listdir(folder)
         if path.isfile(path.join(folder, f))])
 
 def random_FloatField(field):
